from pathlib import Path

__all__ = [
    "CONFIG_PATH_ENV_NAME",
    "QUALIBRATE_CONFIG_KEY",
    "QUALIBRATE_PATH",
    "DEFAULT_CONFIG_FILENAME",
    "DEFAULT_CONFIG_FILEPATH",
    "QUAM_CONFIG_KEY",
    "QUAM_STATE_PATH_CONFIG_KEY",
]
CONFIG_PATH_ENV_NAME = "QUALIBRATE_CONFIG_FILE"

QUALIBRATE_CONFIG_KEY = "qualibrate"
QUAM_CONFIG_KEY = "quam"
QUAM_STATE_PATH_CONFIG_KEY = "state_path"

QUALIBRATE_PATH = Path().home() / ".qualibrate"
DEFAULT_CONFIG_FILENAME = "config.toml"

DEFAULT_CONFIG_FILEPATH = QUALIBRATE_PATH / DEFAULT_CONFIG_FILENAME
